﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace SilverLightPost11434
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;
            Map1.CurrentExtent = new RectangleShape(-135.7, 83.6, 113.5, -53);

            SilverLightTrackOverlay trackOverlay = new SilverLightTrackOverlay(GeographyUnit.DecimalDegree);
            Map1.TrackOverlay = trackOverlay;

            SetupBackOverlay();
        }

        private void SetupBackOverlay()
        {
            WorldMapKitWmsSilverlightOverlay baseOverlay = new WorldMapKitWmsSilverlightOverlay();
            Map1.Overlays.Add(baseOverlay);

            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.State1;
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay inMemoryOverlay = new LayerOverlay();
            inMemoryOverlay.Layers.Add(inMemoryFeatureLayer);
            Map1.Overlays.Add(inMemoryOverlay);
        }

        private void btnTrack_Click(object sender, RoutedEventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                switch (btn.Name)
                {
                    case "btnTrackPolygon":
                        Map1.TrackOverlay.TrackMode = TrackMode.Polygon;
                        break;
                    case "btnTrackLine":
                        Map1.TrackOverlay.TrackMode = TrackMode.Line;
                        break;
                    case "btnTrackNone":
                        Map1.TrackOverlay.TrackMode = TrackMode.None;
                        break;
                }
            }
        }
    }

    public class SilverLightTrackOverlay : TrackInteractiveOverlay
    {
        public delegate void MeasurementChangedEventHandler(object sender, MeasurementChangedEventArgs args);

        public event MeasurementChangedEventHandler measurementChanged;

        private readonly GeographyUnit _mapUnit;

        public SilverLightTrackOverlay(GeographyUnit mapUnit)
            : base()
        {
            this._mapUnit = mapUnit;

            TrackShapeLayer.Open();
            TrackShapeLayer.Columns.Add(new FeatureSourceColumn("length"));

            TrackShapeLayer.ZoomLevelSet = GetZoomLevelSet();
        }

        public Collection<Vertex> Vertexs
        { get { return Vertices; } }

        protected override void DrawCore(RectangleShape worldExtent)
        {
            UpdateTrackingFeature();
            base.DrawCore(worldExtent);
        }

        private void UpdateTrackingFeature()
        {
            if (TrackShapeLayer.InternalFeatures.Any())
            {
                //if (!TrackShapeLayer.InternalFeatures.Any()) return;

                Feature feature = TrackShapeLayer.InternalFeatures.Last();
                BaseShape shape = feature.GetShape();

                MeasurementChangedEventArgs mArgs;

                feature.ColumnValues.Clear();
                var box = shape.GetBoundingBox();

                switch (shape.GetWellKnownType())
                {
                    case WellKnownType.Line:
                        {
                            double length = ((LineShape)shape).GetLength(_mapUnit, DistanceUnit.Meter);
                            feature.ColumnValues.Add("length", string.Format("{0:n0}", (Int64)length) + " Meter");
                            mArgs = new MeasurementChangedEventArgs(length, MeasurementType.Distance);
                        }
                        break;

                    case WellKnownType.Multipolygon:
                        {
                            double area = ((MultipolygonShape)shape).GetArea(_mapUnit, AreaUnit.SquareMeters);
                            feature.ColumnValues.Add("length", string.Format("{0:n0}", (Int64)area) + " Square Meter");
                            mArgs = new MeasurementChangedEventArgs(area, MeasurementType.Area);
                        }
                        break;

                    case WellKnownType.Polygon:
                        {
                            double area = ((PolygonShape)shape).GetArea(_mapUnit, AreaUnit.SquareMeters);
                            feature.ColumnValues.Add("length", string.Format("{0:n0}", (Int64)area) + " Square Meter");
                            mArgs = new MeasurementChangedEventArgs(area, MeasurementType.Area);
                        }
                        break;

                    default:
                        return;
                }
                mArgs.BoundingBox = new Size(box.Width, box.Height);
                if (measurementChanged != null)
                {
                    measurementChanged(this, mArgs);
                }
            }
        }
        public ZoomLevelSet GetZoomLevelSet()
        {
            var style = new ZoomLevelSet();

            var lineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.SimpleColors.Orange, 6.0f, false);
            var areaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Orange);
            var textStyle = TextStyles.CreateMaskTextStyle("length", "Arial", 10f, false, GeoColor.StandardColors.Black, GeoColor.StandardColors.White);

            textStyle.ForceHorizontalLabelForLine = true;

            style.ZoomLevel01.DefaultLineStyle = lineStyle;
            style.ZoomLevel01.DefaultAreaStyle = areaStyle;
            style.ZoomLevel01.DefaultTextStyle = textStyle;
            style.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            return style;
        }
    }

    public class MeasurementChangedEventArgs : EventArgs
    {
        public Size BoundingBox { get; set; }

        /// <summary>
        /// Gets the current measurement amount in meters
        /// </summary>
        public double MetersAmount { get; private set; }

        /// <summary>
        /// Gets the measurement type (area, distance etc.)
        /// </summary>
        public MeasurementType MeasurementType { get; private set; }

        public MeasurementChangedEventArgs(double amount, MeasurementType measurementType)
        {
            this.MetersAmount = amount;
            this.MeasurementType = measurementType;
        }

        public string ToMetricString(bool convert)
        {
            if (convert)
            {
                if (this.MetersAmount > 1000)
                    return this.MeasurementType == MeasurementType.Area ?
                        string.Format("{0:n0}", Math.Round((this.MetersAmount / 1000000), 2)) + " Square km" :
                        string.Format("{0:n0}", Math.Round((this.MetersAmount / 1000), 2)) + " km";
            }

            return string.Format("{0:n0}", Math.Round(this.MetersAmount, 2)) + " m";
        }

        public string ToImperialString(bool convert)
        {
            double amountFeet = this.MetersAmount * 3.2808399;

            if (convert)
            {
                if (amountFeet > 5280)
                    return string.Format("{0:n0}", Math.Round((this.MetersAmount / 5280), 2)) + " mi";
            }

            return string.Format("{0:n0}", Math.Round(this.MetersAmount, 2)) + " ft";
        }
    }

    public enum MeasurementType
    {
        Distance,
        Area
    };
}
